@@ -36,18 +36,18 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp,v 1.24.4.6 2004/07/25 11:12:30 chpe Exp $
+ * $Id: EphyHeaderSniffer.cpp,v 1.36 2004/07/30 19:56:08 chpe Exp $
  */
 
+#include "mozilla-config.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "MozillaPrivate.h"
-
 #include "MozDownload.h"
 #include "EphyHeaderSniffer.h"
-#include "netCore.h"
 
 #include "ephy-file-chooser.h"
 #include "ephy-prefs.h"
@@ -55,24 +55,20 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
-#include "nsReadableUtils.h"
-#include "nsIChannel.h"
-#include "nsIHttpChannel.h"
-#include "nsIURL.h"
-#include "nsIStringEnumerator.h"
-#include "nsIPrefService.h"
-#include "nsIMIMEService.h"
-#include "nsIMIMEInfo.h"
-#include "nsIDOMHTMLDocument.h"
-#include "nsIDownload.h"
-#include "nsIWindowWatcher.h"
-#if MOZILLA_SNAPSHOT > 10
-#include  "nsIMIMEHeaderParam.h"
-#endif
-
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include <nsIChannel.h>
+#include <nsIHttpChannel.h>
+#include <nsIURL.h>
+#include <nsIPrefService.h>
+#include <nsIMIMEService.h>
+#include <nsIMIMEInfo.h>
+#include <nsIDOMHTMLDocument.h>
+#include <nsIDownload.h>
+#include <nsIMIMEHeaderParam.h>
+#include <nsIWindowWatcher.h>
+
 EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmbedPersist *aEmbedPersist,
 		nsIFile* aFile, nsIURI* aURL, nsIDOMDocument* aDocument, nsIInputStream* aPostData,
 		EphyEmbedSingle *single)
@@ -84,10 +80,10 @@ EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmb
 , mDocument(aDocument)
 , mPostData(aPostData)
 {
-	LOG ("EphyHeaderSniffer ctor (%p)", this)
+        LOG ("EphyHeaderSniffer ctor (%p)", this)
 
-	nsCOMPtr<nsIWindowWatcher> watcher
-		(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+        nsCOMPtr<nsIWindowWatcher> watcher
+                (do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
 	if (!watcher) return;
 
 	watcher->GetNewAuthPrompter (nsnull, getter_AddRefs (mAuthPrompt));
@@ -114,7 +110,7 @@ NS_IMETHODIMP
 EphyHeaderSniffer::HandleContent ()
 {
 	gboolean handled = FALSE;
-	nsCString uriSpec;
+	nsEmbedCString uriSpec;
 
 	if (mPostData || !mSingle) return NS_ERROR_FAILURE;
 
@@ -148,7 +144,7 @@ EphyHeaderSniffer::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aReq
 		nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
 		if (httpChannel)
 		{
-			httpChannel->GetResponseHeader(nsCAutoString("content-disposition"),
+			httpChannel->GetResponseHeader(nsEmbedCString("content-disposition"),
 						       mContentDisposition);
 		}
     
@@ -228,7 +224,7 @@ filechooser_response_cb (EphyFileChooser *dialog, gint response, EphyHeaderSniff
 		nsCOMPtr<nsILocalFile> destFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
 		if (destFile)
 		{
-			destFile->InitWithNativePath (nsDependentCString (filename));
+			destFile->InitWithNativePath (nsEmbedCString (filename));
 
 			sniffer->InitiateDownload (destFile);
 		}
@@ -252,70 +248,56 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 	flags = ephy_embed_persist_get_flags (EPHY_EMBED_PERSIST (mEmbedPersist));
 	askDownloadDest = flags & EMBED_PERSIST_ASK_DESTINATION;
 
-	nsAutoString defaultFileName;
+	nsEmbedString defaultFileName;
 
-	if (defaultFileName.IsEmpty() && !mContentDisposition.IsEmpty())
+	if (!defaultFileName.Length() && mContentDisposition.Length())
 	{
 		/* 1 Use the HTTP header suggestion. */
-#if MOZILLA_SNAPSHOT > 10
 		nsCOMPtr<nsIMIMEHeaderParam> mimehdrpar =
 			do_GetService("@mozilla.org/network/mime-hdrparam;1");     
 		
 		if (mimehdrpar)
 		{
-			nsCAutoString fallbackCharset;
+			nsEmbedCString fallbackCharset;
 			if (mURL)
 			{
 				mURL->GetOriginCharset(fallbackCharset);
 			}
 			
-			nsAutoString fileName;
+			nsEmbedString fileName;
 			
 			rv = mimehdrpar->GetParameter (mContentDisposition, "filename",
 						       fallbackCharset, PR_TRUE, nsnull,
 						       fileName);
-			if (NS_FAILED(rv) || fileName.IsEmpty())
+			if (NS_FAILED(rv) || !fileName.Length())
 			{
 				rv = mimehdrpar->GetParameter (mContentDisposition, "name",
 							       fallbackCharset, PR_TRUE, nsnull,
 							       fileName);
 			}
 
-			if (NS_SUCCEEDED(rv) && !fileName.IsEmpty())
+			if (NS_SUCCEEDED(rv) && fileName.Length())
 			{
 				defaultFileName = fileName;
 			}
 		}
-#else
-		PRInt32 index = mContentDisposition.Find("filename=");
-		if (index >= 0)
-		{
-			/* Take the substring following the prefix. */
-			index += strlen ("filename=");
-			nsCAutoString filename;
-			mContentDisposition.Right(filename, mContentDisposition.Length() - index);
-			defaultFileName = NS_ConvertUTF8toUCS2(filename);
-		}
-#endif
 	}
     
-	if (defaultFileName.IsEmpty())
+	if (!defaultFileName.Length())
 	{
 		/* 2 For file URLs, use the file name. */
 
 		nsCOMPtr<nsIURL> url(do_QueryInterface(mURL));
 		if (url)
 		{
-			nsCAutoString fileNameCString;
+			nsEmbedCString fileNameCString;
 			url->GetFileName(fileNameCString);
-			/* FIXME: when we can depend on moz >= 1.5, use
-			 * CopyUTF8toUTF16 instead
-			 */
-			defaultFileName = NS_ConvertUTF8toUCS2(fileNameCString);
+			NS_CStringToUTF16 (fileNameCString, NS_CSTRING_ENCODING_UTF8,
+					   defaultFileName);
 		}
 	}
     
-	if (defaultFileName.IsEmpty() && mDocument)
+	if (!defaultFileName.Length() && mDocument)
 	{
 		/* 3 Use the title of the document. */
 
@@ -326,30 +308,35 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 		}
 	}
     
-	if (defaultFileName.IsEmpty() && mURL)
+	if (!defaultFileName.Length() && mURL)
 	{
 		/* 4 Use the host. */
-		nsCAutoString hostName;
+		nsEmbedCString hostName;
 		mURL->GetHost(hostName);
-		/* FIXME: when we can depend on moz >= 1.5, use
-		 * CopyUTF8toUTF16 instead
-		 */
-		defaultFileName = NS_ConvertUTF8toUCS2(hostName);
+		NS_CStringToUTF16 (hostName, NS_CSTRING_ENCODING_UTF8,
+				   defaultFileName);
 	}
     
 	/* 5 One last case to handle about:blank and other untitled pages. */
-	if (defaultFileName.IsEmpty())
+	if (!defaultFileName.Length())
 	{
-		defaultFileName = NS_ConvertUTF8toUCS2 (_("Untitled"));
+		NS_CStringToUTF16 (nsEmbedCString(_("Untitled")),
+				   NS_CSTRING_ENCODING_UTF8, defaultFileName);
 	}
         
 	/* Validate the file name to ensure legality. */
-	char *default_name = g_strdup (NS_ConvertUCS2toUTF8 (defaultFileName).get());
+	nsEmbedCString cDefaultFileName;
+	NS_UTF16ToCString (defaultFileName, NS_CSTRING_ENCODING_UTF8,
+			   cDefaultFileName);
+	char *default_name = g_strdup (cDefaultFileName.get());
 	default_name = g_strdelimit (default_name, "/\\:", ' ');
 
 	const char *key;
 	key = ephy_embed_persist_get_persist_key (EPHY_EMBED_PERSIST (mEmbedPersist));
 
+	/* FIXME: do better here by using nsITextToSubURI service, like in
+	 * http://lxr.mozilla.org/seamonkey/source/xpfe/communicator/resources/content/contentAreaUtils.js#763
+	 */
         char *filename;
         filename = gnome_vfs_unescape_string (default_name, NULL);
 
@@ -373,7 +360,8 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 		dialog = ephy_file_chooser_new (title ? title: _("Save"),
 						GTK_WIDGET (window),
 						GTK_FILE_CHOOSER_ACTION_SAVE,
-						key ? key : CONF_STATE_SAVE_DIR);
+						key ? key : CONF_STATE_SAVE_DIR,
+						EPHY_FILE_FILTER_ALL_SUPPORTED);
 
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
                                                    filename);
@@ -399,6 +387,7 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 nsresult EphyHeaderSniffer::InitiateDownload (nsILocalFile *aDestFile)
 {
 	LOG ("Initiating download")
+
 	return InitiateMozillaDownload (mDocument, mURL, aDestFile,
 					mContentType.get(), mOriginalURI, mEmbedPersist,
 					mPostData, nsnull, -1);
