@@ -16,9 +16,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphySingle.cpp,v 1.5.4.2 2004/07/25 11:06:43 chpe Exp $
+ *  $Id: EphySingle.cpp,v 1.17 2004/08/21 16:33:58 chpe Exp $
  */
 
+#include "mozilla-config.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -27,37 +29,43 @@
 
 #include "ephy-debug.h"
 
-#include <nsString.h>
+#include <nsEmbedString.h>
 #include <nsIURI.h>
 #include <nsIPermissionManager.h>
 #include <nsICookieManager.h>
 #include <nsIServiceManager.h>
-#if MOZILLA_SNAPSHOT > 9
 #include <nsICookie2.h>
+#include <nsIServiceManager.h>
+
+#ifdef ALLOW_PRIVATE_API
+#include <nsIIDNService.h>
 #endif
 
 NS_IMPL_ISUPPORTS1(EphySingle, nsIObserver)
 
 EphySingle::EphySingle()
+: mOwner(nsnull)
 {
-	LOG ("EphySingle constructor")
-
-	mOwner = nsnull;
+	LOG ("EphySingle ctor")
 }
 
 nsresult
 EphySingle::Init (EphyEmbedSingle *aOwner)
 {
-	LOG ("EphySingle::Init")
-
-	mOwner = aOwner;
-
 	mObserverService = do_GetService ("@mozilla.org/observer-service;1");
 	NS_ENSURE_TRUE (mObserverService, NS_ERROR_FAILURE);
 
-	mObserverService->AddObserver (this, "cookie-changed", PR_FALSE);
-	mObserverService->AddObserver (this, "cookie-rejected", PR_FALSE);
-	mObserverService->AddObserver (this, "perm-changed", PR_FALSE);
+	nsresult rv;
+	rv = mObserverService->AddObserver (this, "cookie-changed", PR_FALSE);
+	rv |= mObserverService->AddObserver (this, "cookie-rejected", PR_FALSE);
+	rv |= mObserverService->AddObserver (this, "perm-changed", PR_FALSE);
+	rv |= mObserverService->AddObserver (this, "network:offline-status-changed", PR_FALSE);
+	rv |= mObserverService->AddObserver (this, "signonChanged", PR_FALSE);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	mOwner = aOwner;
+
+	LOG ("EphySingle::Init")
 
 	return NS_OK;
 }
@@ -65,11 +73,15 @@ EphySingle::Init (EphyEmbedSingle *aOwner)
 nsresult
 EphySingle::Detach ()
 {
+	LOG ("EphySingle::Detach")
+
 	if (mObserverService)
 	{
 		mObserverService->RemoveObserver (this, "cookie-changed");
 		mObserverService->RemoveObserver (this, "cookie-rejected");
 		mObserverService->RemoveObserver (this, "perm-changed");
+		mObserverService->RemoveObserver (this, "signonChanged");
+		mObserverService->RemoveObserver (this, "network:offline-status-changed");
 	}
 
 	return NS_OK;
@@ -77,7 +89,7 @@ EphySingle::Detach ()
 
 EphySingle::~EphySingle()
 {
-	LOG ("EphySingle destructor")
+	LOG ("EphySingle dtor")
 
 	mOwner = nsnull;
 }
@@ -126,6 +138,8 @@ NS_IMETHODIMP EphySingle::Observe(nsISupports *aSubject,
 {
 	nsresult rv = NS_OK;
 
+	LOG ("EphySingle::Observe topic %s", aTopic)
+
 	if (strcmp (aTopic, "cookie-changed") == 0)
 	{
 		/* "added" */
@@ -163,7 +177,7 @@ NS_IMETHODIMP EphySingle::Observe(nsISupports *aSubject,
 		nsCOMPtr<nsIURI> uri = do_QueryInterface (aSubject);
 		if (uri)
 		{
-			nsCAutoString spec;
+			nsEmbedCString spec;
 			uri->GetSpec (spec);
 
 			g_signal_emit_by_name (EPHY_COOKIE_MANAGER (mOwner), "cookie-rejected", spec.get());
@@ -203,24 +217,54 @@ NS_IMETHODIMP EphySingle::Observe(nsISupports *aSubject,
 			rv = NS_ERROR_FAILURE;
 		}
 	}
+	else if (strcmp (aTopic, "signonChanged") == 0)
+	{
+		/* aData can be PRUnichar[] "signons", "rejects", "nocaptures" and "nopreviews" */
+		if (aData[0] == 's')
+		{
+			g_signal_emit_by_name (mOwner, "passwords-changed");
+		}
+	}
+	else if (strcmp (aTopic, "network:offline-status-changed") == 0)
+	{
+		/* aData is either (PRUnichar[]) "offline" or "online" */
+		gboolean offline;
+
+		offline = (aData[1] == 'f');
+
+		g_signal_emit_by_name (mOwner, "network-status", offline);
+	}
 	else
 	{
-		g_warning ("EphySingle observed unknown topic!\n");
+		g_warning ("EphySingle observed unknown topic '%s'!\n", aTopic);
 		rv = NS_ERROR_FAILURE;
 	}
 
+	LOG ("EphySingle::Observe %s", NS_SUCCEEDED (rv) ? "success" : "FAILURE")
+
 	return rv;
 }
 
 EphyCookie *
 mozilla_cookie_to_ephy_cookie (nsICookie *cookie)
 {
-	EphyCookie *info = ephy_cookie_new ();
+	EphyCookie *info;
 
-	nsCAutoString transfer;
+	nsEmbedCString transfer;
 
 	cookie->GetHost (transfer);
-	info->domain = g_strdup (transfer.get());
+
+	nsCOMPtr<nsIIDNService> idnService
+		(do_GetService ("@mozilla.org/network/idn-service;1"));
+	NS_ENSURE_TRUE (idnService, nsnull);
+
+	nsEmbedCString decoded;
+	/* ToUTF8 never fails, no need to check return value */
+	idnService->ConvertACEtoUTF8 (transfer, decoded);
+
+	info = ephy_cookie_new ();
+	info->domain = g_strdup (decoded.get());
+
 	cookie->GetName (transfer);
 	info->name = g_strdup (transfer.get());
 	cookie->GetValue (transfer);
@@ -244,23 +288,19 @@ mozilla_cookie_to_ephy_cookie (nsICookie *cookie)
 	cookie->GetExpires (&dateTime);
 	info->expires = dateTime;
 
-#if MOZILLA_SNAPSHOT > 9
 	nsCOMPtr<nsICookie2> cookie2 = do_QueryInterface (cookie);
-	if (cookie2)
+	NS_ENSURE_TRUE (cookie2, info);
+		
+	PRBool isSession;
+	cookie2->GetIsSession (&isSession);
+	info->is_session = isSession != PR_FALSE;
+		
+	if (!isSession)
 	{
-		
-		PRBool isSession;
-		cookie2->GetIsSession (&isSession);
-		info->is_session = isSession != PR_FALSE;
-		
-		if (!isSession)
-		{
-			PRInt64 expiry;
-			cookie2->GetExpiry (&expiry);
-			info->real_expires = expiry;
-		}
+		PRInt64 expiry;
+		cookie2->GetExpiry (&expiry);
+		info->real_expires = expiry;
 	}
-#endif
 
 	return info;
 }
@@ -268,33 +308,10 @@ mozilla_cookie_to_ephy_cookie (nsICookie *cookie)
 EphyPermissionInfo *
 mozilla_permission_to_ephy_permission (nsIPermission *perm)
 {
-	EphyPermissionType type = (EphyPermissionType) 0;
-
-	nsresult result;
-#if MOZILLA_SNAPSHOT >= 10
-	nsCAutoString str;
-	result = perm->GetType(str);
-	NS_ENSURE_SUCCESS (result, NULL);
-
-	if (str.Equals ("cookie"))
-	{
-		type = EPT_COOKIE;
-	}
-	else if (str.Equals ("image"))
-	{
-		type = EPT_IMAGE;
-	}
-	else if (str.Equals ("popup"))
-	{
-		type = EPT_POPUP;
-	}		
-#else
-	PRUint32 num;
-	result = perm->GetType(&num);
-	NS_ENSURE_SUCCESS (result, NULL);
-
-	type = (EphyPermissionType) num;
-#endif
+	nsresult rv;
+	nsEmbedCString type;
+	rv = perm->GetType(type);
+	NS_ENSURE_SUCCESS (rv, NULL);
 
 	PRUint32 cap;
 	perm->GetCapability(&cap);
@@ -313,8 +330,15 @@ mozilla_permission_to_ephy_permission (nsIPermission *perm)
 			break;
 	}
 
-	nsCString host;
+	nsEmbedCString host;
 	perm->GetHost(host);
 
-	return ephy_permission_info_new (host.get(), type, permission);
+	nsCOMPtr<nsIIDNService> idnService
+		(do_GetService ("@mozilla.org/network/idn-service;1"));
+	NS_ENSURE_TRUE (idnService, nsnull);
+
+	nsEmbedCString decodedHost;
+	idnService->ConvertACEtoUTF8 (host, decodedHost);
+
+	return ephy_permission_info_new (decodedHost.get(), type.get(), permission);
 }
