@@ -16,9 +16,11 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: PrintingPromptService.cpp,v 1.12 2004/01/10 21:59:24 chpe Exp $
+ *  $Id: PrintingPromptService.cpp,v 1.17 2004/07/30 19:56:08 chpe Exp $
  */
 
+#include "mozilla-config.h"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -32,10 +34,10 @@
 #include "PrintingPromptService.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-debug.h"
 
 #include <nsIPrintSettings.h>
 #include <nsCOMPtr.h>
-#include <nsString.h>
 #include <nsIServiceManager.h>
 
 /* Implementation file */
@@ -43,10 +45,12 @@ NS_IMPL_ISUPPORTS1(GPrintingPromptService, nsIPrintingPromptService)
 
 GPrintingPromptService::GPrintingPromptService()
 {
+	LOG ("GPrintingPromptService ctor (%p)", this)
 }
 
 GPrintingPromptService::~GPrintingPromptService()
 {
+	LOG ("GPrintingPromptService dtor (%p)", this)
 }
 
 /* void showPrintDialog (in nsIDOMWindow parent, in nsIWebBrowserPrint webBrowserPrint, in nsIPrintSettings printSettings); */
@@ -68,7 +72,7 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 		GtkWidget *gtkParent = MozillaFindGtkParent(parent);
 		NS_ENSURE_TRUE (gtkParent, NS_ERROR_FAILURE);
 
-		dialog = ephy_print_dialog_new (gtkParent, embed, TRUE);
+		dialog = ephy_print_dialog_new (gtkParent, embed);
 		ephy_dialog_set_modal (dialog, TRUE);
 
 		int ret = ephy_dialog_run (dialog);
@@ -92,7 +96,7 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 		info->pages = 0;
 	}
 
-	MozillaCollatePrintSettings (info, printSettings);
+	MozillaCollatePrintSettings (info, printSettings, FALSE);
 
 	ephy_print_info_free (info);
 
