@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2002-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +15,14 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: egg-toolbars-model.c,v 1.27 2004/07/31 11:55:24 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "egg-toolbars-model.h"
 #include "eggmarshalers.h"
 
@@ -35,6 +42,9 @@ enum
   TOOLBAR_ADDED,
   TOOLBAR_CHANGED,
   TOOLBAR_REMOVED,
+  GET_ITEM_TYPE,
+  GET_ITEM_ID,
+  GET_ITEM_DATA,
   LAST_SIGNAL
 };
 
@@ -51,10 +61,12 @@ typedef struct
   gboolean separator;
 } EggToolbarsItem;
 
-static guint egg_toolbars_model_signals[LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
 
+#define EGG_TOOLBARS_MODEL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EGG_TYPE_TOOLBARS_MODEL, EggToolbarsModelPrivate))
+
 struct EggToolbarsModelPrivate
 {
   GNode *toolbars;
@@ -63,9 +75,9 @@ struct EggToolbarsModelPrivate
 GType
 egg_toolbars_model_get_type (void)
 {
-  static GType egg_toolbars_model_type = 0;
+  static GType type = 0;
 
-  if (egg_toolbars_model_type == 0)
+  if (type == 0)
     {
       static const GTypeInfo our_info = {
 	sizeof (EggToolbarsModelClass),
@@ -79,13 +91,12 @@ egg_toolbars_model_get_type (void)
 	(GInstanceInitFunc) egg_toolbars_model_init
       };
 
-      egg_toolbars_model_type = g_type_register_static (G_TYPE_OBJECT,
-							"EggToolbarsModel",
-							&our_info, 0);
+      type = g_type_register_static (G_TYPE_OBJECT,
+				     "EggToolbarsModel",
+				     &our_info, 0);
     }
 
-  return egg_toolbars_model_type;
-
+  return type;
 }
 
 static xmlDocPtr
@@ -94,7 +105,7 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
   GNode *l1, *l2, *tl;
   xmlDocPtr doc;
 
-  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), NULL);
+  g_return_val_if_fail (EGG_IS_TOOLBARS_MODEL (t), NULL);
 
   tl = t->priv->toolbars;
 
@@ -121,13 +132,13 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
 	    }
 	  else
 	    {
-	      char *name;
+	      char *data;
 
 	      node = xmlNewChild (tnode, NULL, "toolitem", NULL);
-	      name = egg_toolbars_model_get_item_name (t, item->type, item->id);
+	      data = egg_toolbars_model_get_item_data (t, item->type, item->id);
 	      xmlSetProp (node, "type", item->type);
-	      xmlSetProp (node, "name", name);
-	      g_free (name);
+	      xmlSetProp (node, "name", data);
+	      g_free (data);
 	    }
 	}
     }
@@ -135,7 +146,7 @@ egg_toolbars_model_to_xml (EggToolbarsModel *t)
   return doc;
 }
 
-gboolean
+static gboolean
 safe_save_xml (const char *xml_file, xmlDocPtr doc)
 {
 	char *tmp_file;
@@ -199,7 +210,7 @@ egg_toolbars_model_save (EggToolbarsModel *t,
   xmlDocPtr doc;
   xmlNodePtr root;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
 
   doc = egg_toolbars_model_to_xml (t);
   root = xmlDocGetRootElement (doc);
@@ -213,7 +224,7 @@ toolbars_toolbar_new (const char *name)
 {
   EggToolbarsToolbar *toolbar;
 
-  toolbar = g_new0 (EggToolbarsToolbar, 1);
+  toolbar = g_new (EggToolbarsToolbar, 1);
   toolbar->name = g_strdup (name);
   toolbar->flags = 0;
 
@@ -230,7 +241,7 @@ toolbars_item_new (const char *id,
   g_return_val_if_fail (id != NULL, NULL);
   g_return_val_if_fail (type != NULL, NULL);
 
-  item = g_new0 (EggToolbarsItem, 1);
+  item = g_new (EggToolbarsItem, 1);
   item->id = g_strdup (id);
   item->type = g_strdup (type);
   item->separator = separator;
@@ -239,22 +250,26 @@ toolbars_item_new (const char *id,
 }
 
 static void
-free_toolbar_node (EggToolbarsToolbar *toolbar)
+free_toolbar_node (GNode *toolbar_node)
 {
-  g_return_if_fail (toolbar != NULL);
+  EggToolbarsToolbar *toolbar = toolbar_node->data;
 
   g_free (toolbar->name);
   g_free (toolbar);
+
+  g_node_destroy (toolbar_node);
 }
 
 static void
-free_item_node (EggToolbarsItem *item)
+free_item_node (GNode *item_node)
 {
-  g_return_if_fail (item != NULL);
+  EggToolbarsItem *item = item_node->data;
 
   g_free (item->id);
   g_free (item->type);
   g_free (item);
+
+  g_node_destroy (item_node);
 }
 
 EggTbModelFlags
@@ -265,7 +280,7 @@ egg_toolbars_model_get_flags (EggToolbarsModel *t,
   EggToolbarsToolbar *toolbar;
 
   toolbar_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  g_return_val_if_fail (toolbar_node != NULL, -1);
+  g_return_val_if_fail (toolbar_node != NULL, 0);
 
   toolbar = toolbar_node->data;
 
@@ -274,8 +289,8 @@ egg_toolbars_model_get_flags (EggToolbarsModel *t,
 
 void
 egg_toolbars_model_set_flags (EggToolbarsModel *t,
-			      EggTbModelFlags   flags,
-			      int               toolbar_position)
+			      int               toolbar_position,
+			      EggTbModelFlags   flags)
 {
   GNode *toolbar_node;
   EggToolbarsToolbar *toolbar;
@@ -287,7 +302,7 @@ egg_toolbars_model_set_flags (EggToolbarsModel *t,
 
   toolbar->flags = flags;
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_CHANGED],
+  g_signal_emit (G_OBJECT (t), signals[TOOLBAR_CHANGED],
 		 0, toolbar_position);
 }
 
@@ -301,20 +316,20 @@ egg_toolbars_model_add_separator (EggToolbarsModel *t,
   EggToolbarsItem *item;
   int real_position;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
 
   parent_node = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  item = toolbars_item_new ("separator", "separator", TRUE);
+  item = toolbars_item_new ("separator", EGG_TOOLBAR_ITEM_TYPE, TRUE);
   node = g_node_new (item);
   g_node_insert (parent_node, position, node);
 
   real_position = g_node_child_position (parent_node, node);
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[ITEM_ADDED], 0,
+  g_signal_emit (G_OBJECT (t), signals[ITEM_ADDED], 0,
 		 toolbar_position, real_position);
 }
 
-gboolean
+static gboolean
 impl_add_item (EggToolbarsModel    *t,
 	       int		    toolbar_position,
 	       int		    position,
@@ -326,7 +341,7 @@ impl_add_item (EggToolbarsModel    *t,
   EggToolbarsItem *item;
   int real_position;
 
-  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), FALSE);
+  g_return_val_if_fail (EGG_IS_TOOLBARS_MODEL (t), FALSE);
   g_return_val_if_fail (id != NULL, FALSE);
   g_return_val_if_fail (type != NULL, FALSE);
 
@@ -337,7 +352,7 @@ impl_add_item (EggToolbarsModel    *t,
 
   real_position = g_node_child_position (parent_node, node);
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[ITEM_ADDED], 0,
+  g_signal_emit (G_OBJECT (t), signals[ITEM_ADDED], 0,
 		 toolbar_position, real_position);
 
   return TRUE;
@@ -359,17 +374,20 @@ parse_item_list (EggToolbarsModel *t,
 	  type = xmlGetProp (child, "type");
           if (type == NULL)
             {
-              type = g_strdup (EGG_TOOLBAR_ITEM_TYPE);
+              type = xmlStrdup (EGG_TOOLBAR_ITEM_TYPE);
             }
 
-          id = egg_toolbars_model_get_item_id (t, type, name);
-	  if (id != NULL)
+	  if (name != NULL && name[0] != '\0' && type != NULL)
 	    {
-	      egg_toolbars_model_add_item (t, position, -1, id, type);
+              id = egg_toolbars_model_get_item_id (t, type, name);
+	      if (id != NULL)
+	        {
+	          egg_toolbars_model_add_item (t, position, -1, id, type);
+                }
+              g_free (id);
             }
 	  xmlFree (name);
-          g_free (type);
-          g_free (id);
+          xmlFree (type);
 	}
       else if (xmlStrEqual (child->name, "separator"))
 	{
@@ -388,14 +406,14 @@ egg_toolbars_model_add_toolbar (EggToolbarsModel *t,
   GNode *node;
   int real_position;
 
-  g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), -1);
+  g_return_val_if_fail (EGG_IS_TOOLBARS_MODEL (t), -1);
 
   node = g_node_new (toolbars_toolbar_new (name));
   g_node_insert (t->priv->toolbars, position, node);
 
   real_position = g_node_child_position (t->priv->toolbars, node);
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_ADDED],
+  g_signal_emit (G_OBJECT (t), signals[TOOLBAR_ADDED],
 		 0, real_position);
 
   return g_node_child_position (t->priv->toolbars, node);
@@ -410,12 +428,21 @@ parse_toolbars (EggToolbarsModel *t,
       if (xmlStrEqual (child->name, "toolbar"))
 	{
 	  xmlChar *name;
+	  xmlChar *style;
 	  int position;
 
 	  name = xmlGetProp (child, "name");
 	  position = egg_toolbars_model_add_toolbar (t, -1, name);
 	  xmlFree (name);
 
+	  style = xmlGetProp (child, "style");
+	  if (style && xmlStrEqual (style, "icons-only"))
+	    {
+	      /* FIXME: use toolbar position instead of 0 */
+	      egg_toolbars_model_set_flags (t, 0, EGG_TB_MODEL_ICONS_ONLY);
+	    }
+	  xmlFree (style);
+
 	  parse_item_list (t, child->children, position);
 	}
 
@@ -423,44 +450,47 @@ parse_toolbars (EggToolbarsModel *t,
     }
 }
 
-void
+gboolean
 egg_toolbars_model_load (EggToolbarsModel *t,
 			 const char *xml_file)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_val_if_fail (EGG_IS_TOOLBARS_MODEL (t), FALSE);
+
+  if (!xml_file || !g_file_test (xml_file, G_FILE_TEST_EXISTS)) return FALSE;
 
   doc = xmlParseFile (xml_file);
   if (doc == NULL)
   {
     g_warning ("Failed to load XML data from %s", xml_file);
-    return;
+    return FALSE;
   }
   root = xmlDocGetRootElement (doc);
 
-  t->priv->toolbars = g_node_new (NULL);
   parse_toolbars (t, root->children);
 
   xmlFreeDoc (doc);
+
+  return TRUE;
 }
 
-char *
+static char *
 impl_get_item_id (EggToolbarsModel *t,
 		  const char       *type,
-		  const char       *name)
+		  const char       *data)
 {
   if (strcmp (type, EGG_TOOLBAR_ITEM_TYPE) == 0)
     {
-      return g_strdup (name);
+      return g_strdup (data);
     }
 
   return NULL;
 }
 
-char *
-impl_get_item_name (EggToolbarsModel *t,
+static char *
+impl_get_item_data (EggToolbarsModel *t,
 		    const char       *type,
 		    const char       *id)
 {
@@ -472,7 +502,7 @@ impl_get_item_name (EggToolbarsModel *t,
   return NULL;
 }
 
-char *
+static char *
 impl_get_item_type (EggToolbarsModel *t,
 		    GdkAtom type)
 {
@@ -484,6 +514,23 @@ impl_get_item_type (EggToolbarsModel *t,
   return NULL;
 }
 
+static gboolean
+_egg_accumulator_STRING (GSignalInvocationHint *ihint,
+                         GValue                *return_accu,
+                         const GValue          *handler_return,
+                         gpointer               dummy)
+{
+  gboolean continue_emission;
+  const char *retval;
+
+  retval = g_value_get_string (handler_return);
+  g_value_set_string (return_accu, retval);
+  continue_emission = !retval || !retval[0];
+  
+  return continue_emission;
+}
+
+
 static void
 egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 {
@@ -495,52 +542,86 @@ egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 
   klass->add_item = impl_add_item;
   klass->get_item_id = impl_get_item_id;
-  klass->get_item_name = impl_get_item_name;
+  klass->get_item_data = impl_get_item_data;
   klass->get_item_type = impl_get_item_type;
 
-  egg_toolbars_model_signals[ITEM_ADDED] =
+  signals[ITEM_ADDED] =
     g_signal_new ("item_added",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, item_added),
 		  NULL, NULL, _egg_marshal_VOID__INT_INT,
 		  G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
-  egg_toolbars_model_signals[TOOLBAR_ADDED] =
+  signals[TOOLBAR_ADDED] =
     g_signal_new ("toolbar_added",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_added),
 		  NULL, NULL, g_cclosure_marshal_VOID__INT,
 		  G_TYPE_NONE, 1, G_TYPE_INT);
-  egg_toolbars_model_signals[ITEM_REMOVED] =
+  signals[ITEM_REMOVED] =
     g_signal_new ("item_removed",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, item_removed),
 		  NULL, NULL, _egg_marshal_VOID__INT_INT,
 		  G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
-  egg_toolbars_model_signals[TOOLBAR_REMOVED] =
+  signals[TOOLBAR_REMOVED] =
     g_signal_new ("toolbar_removed",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_removed),
 		  NULL, NULL, g_cclosure_marshal_VOID__INT,
 		  G_TYPE_NONE, 1, G_TYPE_INT);
-  egg_toolbars_model_signals[TOOLBAR_CHANGED] =
+  signals[TOOLBAR_CHANGED] =
     g_signal_new ("toolbar_changed",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_changed),
 		  NULL, NULL, g_cclosure_marshal_VOID__INT,
 		  G_TYPE_NONE, 1, G_TYPE_INT);
+  signals[GET_ITEM_TYPE] =
+    g_signal_new ("get_item_type",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_type),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__POINTER,
+		  G_TYPE_STRING, 1, G_TYPE_POINTER);
+  signals[GET_ITEM_ID] =
+    g_signal_new ("get_item_id",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_id),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__STRING_STRING,
+		  G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);
+  signals[GET_ITEM_DATA] =
+    g_signal_new ("get_item_data",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, get_item_data),
+		  _egg_accumulator_STRING, NULL,
+		  _egg_marshal_STRING__STRING_STRING,
+		  G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);
+
+  g_type_class_add_private (object_class, sizeof (EggToolbarsModelPrivate));
 }
 
 static void
 egg_toolbars_model_init (EggToolbarsModel *t)
 {
-  t->priv = g_new0 (EggToolbarsModelPrivate, 1);
+  t->priv =EGG_TOOLBARS_MODEL_GET_PRIVATE (t);
 
-  t->priv->toolbars = NULL;
+  t->priv->toolbars = g_node_new (NULL);
+}
+
+static void
+free_toolbar (GNode *toolbar_node)
+{
+  g_node_children_foreach (toolbar_node, G_TRAVERSE_ALL,
+    			   (GNodeForeachFunc) free_item_node, NULL);
+  free_toolbar_node (toolbar_node);
 }
 
 static void
@@ -548,27 +629,17 @@ egg_toolbars_model_finalize (GObject *object)
 {
   EggToolbarsModel *t = EGG_TOOLBARS_MODEL (object);
 
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (object));
-
-  /* FIXME free nodes */
+  g_node_children_foreach (t->priv->toolbars, G_TRAVERSE_ALL,
+    			   (GNodeForeachFunc) free_toolbar, NULL);
   g_node_destroy (t->priv->toolbars);
 
-  g_free (t->priv);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 EggToolbarsModel *
 egg_toolbars_model_new (void)
 {
-  EggToolbarsModel *t;
-
-  t = EGG_TOOLBARS_MODEL (g_object_new (EGG_TOOLBARS_MODEL_TYPE, NULL));
-
-  g_return_val_if_fail (t->priv != NULL, NULL);
-
-  return t;
+  return EGG_TOOLBARS_MODEL (g_object_new (EGG_TYPE_TOOLBARS_MODEL, NULL));
 }
 
 void
@@ -578,19 +649,18 @@ egg_toolbars_model_remove_toolbar (EggToolbarsModel   *t,
   GNode *node;
   EggTbModelFlags flags;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
 
   flags = egg_toolbars_model_get_flags (t, position);
 
-  if (!(flags && EGG_TB_MODEL_NOT_REMOVABLE))
+  if (!(flags & EGG_TB_MODEL_NOT_REMOVABLE))
     {
       node = g_node_nth_child (t->priv->toolbars, position);
       g_return_if_fail (node != NULL);
 
-      free_toolbar_node (node->data);
-      g_node_destroy (node);
+      free_toolbar_node (node);
 
-      g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_REMOVED],
+      g_signal_emit (G_OBJECT (t), signals[TOOLBAR_REMOVED],
 		     0, position);
     }
 }
@@ -602,7 +672,7 @@ egg_toolbars_model_remove_item (EggToolbarsModel *t,
 {
   GNode *node, *toolbar;
 
-  g_return_if_fail (IS_EGG_TOOLBARS_MODEL (t));
+  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
 
   toolbar = g_node_nth_child (t->priv->toolbars, toolbar_position);
   g_return_if_fail (toolbar != NULL);
@@ -610,13 +680,43 @@ egg_toolbars_model_remove_item (EggToolbarsModel *t,
   node = g_node_nth_child (toolbar, position);
   g_return_if_fail (node != NULL);
 
-  free_item_node (node->data);
-  g_node_destroy (node);
+  free_item_node (node);
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[ITEM_REMOVED], 0,
+  g_signal_emit (G_OBJECT (t), signals[ITEM_REMOVED], 0,
 		 toolbar_position, position);
 }
 
+void
+egg_toolbars_model_move_item (EggToolbarsModel *t,
+			      int               toolbar_position,
+			      int               position,
+			      int		new_toolbar_position,
+			      int		new_position)
+{
+  GNode *node, *toolbar, *new_toolbar;
+
+  g_return_if_fail (EGG_IS_TOOLBARS_MODEL (t));
+
+  toolbar = g_node_nth_child (t->priv->toolbars, toolbar_position);
+  g_return_if_fail (toolbar != NULL);
+
+  new_toolbar = g_node_nth_child (t->priv->toolbars, new_toolbar_position);
+  g_return_if_fail (new_toolbar != NULL);
+
+  node = g_node_nth_child (toolbar, position);
+  g_return_if_fail (node != NULL);
+
+  g_node_unlink (node);
+
+  g_signal_emit (G_OBJECT (t), signals[ITEM_REMOVED], 0,
+		 toolbar_position, position);
+
+  g_node_insert (new_toolbar, new_position, node);
+
+  g_signal_emit (G_OBJECT (t), signals[ITEM_ADDED], 0,
+		 new_toolbar_position, new_position);
+}
+
 int
 egg_toolbars_model_n_items (EggToolbarsModel *t,
 			    int               toolbar_position)
@@ -629,27 +729,37 @@ egg_toolbars_model_n_items (EggToolbarsModel *t,
   return g_node_n_children (toolbar);
 }
 
-const char *
+void
 egg_toolbars_model_item_nth (EggToolbarsModel *t,
 			     int	       toolbar_position,
 			     int               position,
-			     gboolean         *is_separator)
+			     gboolean         *is_separator,
+			     const char      **id,
+			     const char      **type)
 {
   GNode *toolbar;
   GNode *item;
   EggToolbarsItem *idata;
 
   toolbar = g_node_nth_child (t->priv->toolbars, toolbar_position);
-  g_return_val_if_fail (toolbar != NULL, NULL);
+  g_return_if_fail (toolbar != NULL);
 
   item = g_node_nth_child (toolbar, position);
-  g_return_val_if_fail (item != NULL, NULL);
+  g_return_if_fail (item != NULL);
 
   idata = item->data;
 
   *is_separator = idata->separator;
 
-  return idata->id;
+  if (id)
+    {
+      *id = idata->id;
+    }
+
+  if (type)
+    {
+      *type = idata->type;
+    }
 }
 
 int
@@ -689,23 +799,32 @@ egg_toolbars_model_get_item_id (EggToolbarsModel *t,
 			        const char       *type,
 			        const char       *name)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_id (t, type, name);
+  char *retval;
+
+  g_signal_emit (t, signals[GET_ITEM_ID], 0, type, name, &retval);
+
+  return retval;
 }
 
 char *
-egg_toolbars_model_get_item_name (EggToolbarsModel *t,
+egg_toolbars_model_get_item_data (EggToolbarsModel *t,
 				  const char       *type,
 			          const char       *id)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_name (t, type, id);
+  char *retval;
+
+  g_signal_emit (t, signals[GET_ITEM_DATA], 0, type, id, &retval);
+
+  return retval;
 }
 
 char *
 egg_toolbars_model_get_item_type (EggToolbarsModel *t,
 				  GdkAtom type)
 {
-  EggToolbarsModelClass *klass = EGG_TOOLBARS_MODEL_GET_CLASS (t);
-  return klass->get_item_type (t, type);
+  char *retval;
+
+  g_signal_emit (t, signals[GET_ITEM_TYPE], 0, type, &retval);
+
+  return retval;
 }
