@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.203 2004/08/06 14:34:07 chpe Exp $
+ *  $Id: ephy-window.c,v 1.204 2004/08/21 16:31:49 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -180,7 +180,7 @@ static GtkActionEntry ephy_menu_entries [] = {
 	{ "ViewZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "<control>minus",
 	  N_("Decrease the text size"),
 	  G_CALLBACK (window_cmd_view_zoom_out) },
-	{ "ViewZoomNormal", GTK_STOCK_ZOOM_100, N_("_Normal Size"), NULL,
+	{ "ViewZoomNormal", GTK_STOCK_ZOOM_100, N_("_Normal Size"), "<control>0",
 	  N_("Use the normal text size"),
 	  G_CALLBACK (window_cmd_view_zoom_normal) },
 	{ "ViewEncoding", NULL, N_("Text _Encoding"), NULL,
